package WT::auth;

# library of permissions-handling routines for WebThread
# $Id: auth.pm,v 1.2.1.1 1996/09/06 06:05:22 dmacks Rel $

# WebThread, a hacked together cgi-bin thingy by Daniel Macks
# This file copyright (c) 1996 by Emaze Software Corp. and Daniel Macks.
# All rights reserved. See the file COPYRITE for more information.

#use strict;
#no strict qw(subs);	# otherwise barfs on O_* and SDBM_File

use WT::errors;
use Fcntl qw(O_CREAT O_RDWR O_RDONLY);
use SDBM_File;
use flocker;

### constructor ###
# define filenames
# define the bits of the auth vector
###################
$WT::auth::dbperms=$main::dbperms;
sub import {	# first argument to use/import is full path to dir with auth
	$WT::auth::authDB="$_[1]${main::SL}auth";
	unless (&flocker::NewLockfile($WT::auth::authDB)) {
		&main::ErrServ("cannot secure authDB");
		&main::LogWarn($flocker::msg);
		defined &main::End ? &main::End : exit 1;
	};
};

## 0	"access"	can use the system
## 1	"read"		can read threads		/read
## 2	"post"		can post messages		/post
## 3	"new"		can spawn/begin new threads	/post
## 4	"unpost"	can unpost own messages		/unpost
## 5	"user"		can modify user info		/user
## 6	"admin"		can manage threads		/admin /ticklist
## 7	"superuser"	can change other users' info	/superuser /ticklist
## 8	"adduser"	can add new users		/newuser

@WT::auth::authflags=("access","read","post","new","unpost","user","admin","superuser","adduser");

### public routines ###
# returns string (hex number) for all auth bits set
#######################
sub main::FullAuth {
	&main::AuthNumFromList(@WT::auth::authflags);
}

### public routines ###
# convert list of flag names to string (hex number)
#######################
sub main::AuthNumFromList {
	my(@authreq)=@_;
	my(%authreq,$reqflag);

	while ($reqflag=(shift @authreq)) {
		$authreq{$reqflag}=1;
	};

	my $authvec = "\0";
	my($bit);
	for($bit=0;$bit<@WT::auth::authflags;$bit++) {
		vec($authvec,$bit,1)=1 if $authreq{$WT::auth::authflags[$bit]};
	};

	unpack("H*",$authvec);
}

### public routines ###
# masks the string (hex number) ($_[1]) for the given uid ($_[0])
#######################
sub main::MaskAuth {
	my($uid,$oldauthstring)=@_;

	if ($uid==1) {		# master can always do everything
		&main::FullAuth;
	} else {		# everyone else subject to masks
		my($oldauthvec,$givevec,$denyvec,$newauthvec);
		$oldauthvec=pack("H*",$oldauthstring);		# 1=allow
		$givevec=pack("H*",&main::GetAuth('give'));	# 1=allow
		$denyvec=pack("H*",&main::GetAuth('deny'));	# 1=deny

		$denyvec= $denyvec ^ pack("H*",&main::FullAuth);	# 1=allow

		$newauthvec= ( $oldauthvec | $givevec ) & $denyvec;
		unpack("H*",$newauthvec);
	};
}

### public routines ###
# convert string (hex number) into associative array
#######################
sub main::AuthAAFromNum {
	my($authvec)=pack("H*",$_[0]);
	my(%userauth);
	my($bit);
	for($bit=0;$bit<@WT::auth::authflags;$bit++) {
		$userauth{$WT::auth::authflags[$bit]}=vec($authvec,$bit,1);
	};
	%userauth;
}		

### public routines ###
# get auth for a user as string (hex number)
#######################
sub main::GetAuth {		# returns auth for uid=$_[0]
	my($uid)=$_[0];
	my(%authDB);
	&flocker::flockSH($WT::auth::authDB);
	unless (tie %authDB,SDBM_File,$WT::auth::authDB,O_RDONLY,0) {
		&flocker::flockUN($WT::auth::authDB);
		&main::ErrServ("cannot access authDB");
		&main::LogWarn("could not access $WT::auth::authDB: $!");
		&main::End;
	};
	my($authstring)=$authDB{$uid};
	untie %authDB;
	&flocker::flockUN($WT::auth::authDB);
	$authstring;
}	

### public routines ###
# set auth for a user to string (hex number)
#######################
sub main::SetAuth {	# sets uid=$_[0] to auth=$_[1]
	my($uid,$authstring)=@_;
	my(%authDB);
	&flocker::flockEX($WT::auth::authDB);
	unless (tie %authDB,SDBM_File,$WT::auth::authDB,O_CREAT|O_RDWR,$WT::auth::dbperms) {
		&flocker::flockUN($WT::auth::authDB);
		&main::ErrServ("cannot access authDB");
		&main::LogWarn("could not access $WT::auth::authDB: $!");
		&main::End;
	};
	$authDB{$uid}=$authstring;
	untie %authDB;
	&flocker::flockUN($WT::auth::authDB);
}	

### public routines ###
# lists granted auths (or checkboxes if $_[0] is true) for uid=$_[1]
#######################
sub main::OutputAuthFlags {
	my($isSU,$authstring)=( $_[0] , &main::GetAuth($_[1]) );
	my(%userauth)=&main::AuthAAFromNum($authstring);
	my($flag);

	print "   auth:      ";
	if ($isSU) {
		print "click to change:\n";
	} else {
		print "contact a superuser to change:\n";
	};

	foreach $flag (@WT::auth::authflags) {
		if ($isSU) {
			print " "x18;
			print "<input type=checkbox name=CBauth value=$flag";
			print " checked" if $userauth{$flag};
			print "> $flag\n";
		} elsif ($userauth{$flag}) {
			print " "x18;
			print "+<input type=hidden name=CBauth value=$flag> $flag\n";
		};
	};
}

### public routines ###
# lists table of auths for uids @_
#######################
sub main::OutputManyAuthFlags {
	my(@targetuids,$targetuid,$uname,%userauth,$flag);
	@targetuids=@_;

	print "<center><table border><tr><th>username<br>";
	foreach $flag (@WT::auth::authflags) {
		print "<th>$flag<br>";
	};

	foreach $targetuid (@targetuids) {
		$uname=&main::LookupUname($targetuid);
		%userauth=&main::AuthAAFromNum(&main'GetAuth($targetuid));

		print "<tr><td align=left>$uname";
		print "<input type=hidden name=targetuid value=$targetuid>";

		foreach $flag (@WT::auth::authflags) {
			print "<td align=center><input type=checkbox name=CBauth$targetuid value=$flag";
			print " checked" if $userauth{$flag};
			print ">";
		};
		print "<br>\n";
	};
	print "</table></center>\n";
}

### public routines ###
# output an html table of auth flag options
#######################
sub main::OutputSUAuth {
	my(%default) = &main::AuthAAFromNum( &main::GetAuth( 'default' ));
	my(%give)    = &main::AuthAAFromNum( &main::GetAuth( 'give'    ));
	my(%deny)    = &main::AuthAAFromNum( &main::GetAuth( 'deny'    ));
	my($flag);

	print "<center><table border><tr><td><td>";
	foreach $flag (@WT::auth::authflags) {
		print "<th>$flag</th>";
	};

	print "<tr><th rowspan=3>m<br>a<br>s<br>k<br></th><th>give<br></th>\n";
	foreach $flag (@WT::auth::authflags) {
		print "<td align=center><input type=radio name=mask$flag value=g";
		print " checked" if $give{$flag};
		print ">";
	};

	print "<tr><th>user<br></th>\n";
	foreach $flag (@WT::auth::authflags) {
		print "<td align=center><input type=radio name=mask$flag value=u";
		print " checked" unless $give{$flag} || $deny{$flag};
		print ">";
	};

	print "<tr><th>deny<br></th>\n";
	foreach $flag (@WT::auth::authflags) {
		print "<td align=center><input type=radio name=mask$flag value=d";
		print " checked" if $deny{$flag};
		print ">";
	};

	print "<tr><th colspan=2>new user<br>default<br></th>\n";
	foreach $flag (@WT::auth::authflags) {
		print "<td align=center><input type=checkbox name=CBdefault value=$flag";
		print " checked" if $default{$flag};
		print ">";
	};

	print "</table>\n";
}

### public routines ###
# set $in{$_[0]} according to checkboxes (\0-delim multivalue) $in{"CB$_[0]"}
#######################
sub main::SetNumFromCB {
	my($field)=$_[0];
	my(@request)=split("\0",$main::in{"CB$field"});
	$main::in{$field}=&main::AuthNumFromList(@request);
}

### public routines ###
# process the respose to superuser auth def changes
#######################
sub main::SetAuthDefs {
	&main::SetNumFromCB('default');			# new-user defaults
	&main::SetAuth('default',$main::in{'default'});

	my($flag,@give,@deny);
	foreach $flag (@WT::auth::authflags) {
		push(@give,$flag) if $main::in{"mask$flag"} eq 'g';
		push(@deny,$flag) if $main::in{"mask$flag"} eq 'd';
	};
	&main::SetAuth('give',&main::AuthNumFromList(@give));
	&main::SetAuth('deny',&main::AuthNumFromList(@deny));
}

1;
