package WT::errors;

# library of error-handling routines for WebThread
# $Id: errors.pm,v 1.2 1996/09/21 18:45:54 dmacks Rel $

# WebThread, a hacked together cgi-bin thingy by Daniel Macks
# This file copyright (c) 1996 by Emaze Software Corp. and Daniel Macks.
# All rights reserved. See the file COPYRITE for more information.

#use strict;

# $main::errorLog determines what happens to error messages
#    (none)   -> STDERR
#    "ignore" -> /dev/null
#    /logfile -> /logfile
$SIG{'__WARN__'} = sub { &WT::errors::ErrorLog('SIG__WARN__',$_[0]); };

# keep time handy, with consistent format, for all error msgs
$WT::errors::now = sprintf "%2.1d/%2.1d/%.2d %2.1d:%.2d:%.2d",
	(localtime($^T))[4,3,5,2,1,0];

### public routines ###
# return various error messages to the user
#######################

sub main::ErrUser {
	&WT::errors::ErrorRpt("WebThread user error","use your browser's &quot;back&quot; button and fix the following error:",@_);
};

sub main::ErrServ {
	&WT::errors::ErrorRpt("WebThread server error","the following fatal WebThread error has occured:",@_);
};

sub main::ErrHack {
	&WT::errors::ErrorRpt("WebThread authentication error","that action is not allowed for the following reason:",@_);
}

### public routines ###
# log various error messages
#######################

sub main::LogWarn {		# fatal system errors
	push (@_,"(no details available)") unless @_;
	&WT::errors::ErrorLog("error: fatal",@_);
}

sub main::LogHack {		# things that look like hack attempts
	push (@_,"(no details available)") unless @_;
	&WT::errors::ErrorLog("warning: possible crack attempt from $ENV{'REMOTE_ADDR'}",@_);
}

sub main::LogInOut {		# tracks logins/logouts (det. by $_[0])
	my $uname = &main::LookupUname($main::uid);
	if ($main::trackUse==1) {
		&WT::errors::ErrorLog("user: $_[0]","$uname");
	} elsif ($main::trackUse==2) {
		open(USERLOG,">>${main::treeDIR}${main::SL}user-log");
		print USERLOG "$WT::errors::now (pid $$, uid $main::uid): $_[0]: $uname\n";
	};
}

### private routines ###
# actually do the logging
########################
sub WT::errors::ErrorLog {	# log the error according to $main::errorLog
	my $header = "WebThread \"$main::lnf_name\" ".shift;
	my($cause,$line);

	return if $main::errorLog=~/^ignore$/i;
	open(ERRORLOG,">>$ENV{'projectDIR'}${main::SL}${main::errorLog}") if $main::errorLog;
	while ($cause=shift) {
		$line="$WT::errors::now (pid $$, uid $main::uid): $header: $cause";
		$main::errorLog ? print ERRORLOG "$line\n" : print main::STDERR "$line\n";
	};
	close(ERRORLOG) if $main::errorLog;
}

### private routines ###
# generate html-formatted error report on STDOUT
########################
sub WT::errors::ErrorRpt {
	my $header = shift || '';
	my $instr  = shift || '';
	my $cause;
	my $CRLF = "\015\012";

	print "Status: 200 OK$CRLF";
	print "Content-type: text/html$CRLF";
	print "$CRLF";
	print "<html><head><title>$header</title></head>\n";
	print "<center><h1>$header</h1></center>\n";
	print "$instr\n<ul>\n";
	while ($cause = shift @_) {
		print "<li><strong>$cause</strong>\n";
	};
	print "</ul></body></html>";
}


1;
