package WT::pass;

# library of password routines for WebThread
# $Id: pass.pm,v 2.1.1.1 1996/10/06 19:54:36 dmacks Rel $

# WebThread, a hacked together cgi-bin thingy by Daniel Macks
# This file copyright (c) 1996 by Emaze Software Corp. and Daniel Macks.
# All rights reserved. See the file COPYRITE for more information.

#use strict;
#no strict qw(subs);	# otherwise barfs on O_* and SDBM_File

use WT::errors;
use Fcntl qw(O_CREAT O_RDWR O_RDONLY);
use SDBM_File;
use flocker;

### constructor ###
# define filenames and other constants
###################
$WT::pass::dbperms=$main::dbperms;
sub import {	# first argument to use/import is full path to dir with pass
	$WT::pass::passDB="$_[1]${main::SL}pass";
	unless (&flocker::NewLockfile($WT::pass::passDB)) {
		&main::ErrServ("cannot secure passDB");
		&main::LogWarn($flocker::msg);
		defined &main::End ? &main::End : exit 1;
	};
};

# %passDB:
#	count\0		highest uid	number of users
#	uid\^[uname]\0	uname->uid	uid for user [uname]
#	uname\^[uid]\0	uid->uname	uname for [uid]	
#	pw\^[uid]\0	uid->password	encrypted password for [uid]

$WT::pass::iter=0;	# iterator for EachUser

### public routines ###
# checks a user's password
#######################
sub main::PWCheck {	# checks that uname $_[0] has password $_[1]
	my($uname,$pw,$action)=@_;
	my(%passDB,$uid);

	&flocker::flockSH($WT::pass::passDB);
	unless (tie %passDB,SDBM_File,$WT::pass::passDB,O_RDONLY,0) {
		&main::ErrServ("cannot access passDB");
		&main::LogWarn("could not access passDB: $!");
		&main::End;
	};

	unless ($uid=$passDB{"uid","$uname\0"}) {
		untie %passDB;
		&flocker::flockUN($WT::pass::passDB);
		&main::ErrHack("invalid login");
		&main::LogHack("unknown user ($uname) tried to $action");
		&main::End;
	};

	if (crypt($pw,$passDB{"pw","$uid\0"}) ne $passDB{"pw","$uid\0"}) {
		untie %passDB;
		&flocker::flockUN($WT::pass::passDB);
		&main::ErrHack("incorrect password");
		&main::LogHack("incorrect password entered for $action by $uname");
		&main::End;
	};

	untie %passDB;
	&flocker::flockUN($WT::pass::passDB);
}

### public routines ###
# sets a user's username
#######################
sub main::ChangeUname {	# sets the username for uid $_[0] to $_[1]
	my($uid,$uname)=@_;
	my(%passDB);

	&flocker::flockEX($WT::pass::passDB);
	unless (tie %passDB,SDBM_File,$WT::pass::passDB,O_RDWR,$WT::pass::dbperms) {
		&flocker::flockUN($WT::pass::passDB);
		&main::ErrServ("cannot access passDB");
		&main::LogWarn("could not access passDB: $!");
		&main::End;
	};

	my $olduname = $passDB{"uname","$uid\0"};
	delete $passDB{"uid","$olduname\0"};

	$passDB{"uname","$uid\0"}=$uname;
	$passDB{"uid","$uname\0"}=$uid;

	untie %passDB;
	&flocker::flockUN($WT::pass::passDB);
}

### public routines ###
# sets a user's password
#######################
sub main::PWSet {	# sets the password for uid $_[0] to $_[1]
	my($uid,$pw)=@_;
	my(%passDB);

	&flocker::flockEX($WT::pass::passDB);
	unless (tie %passDB,SDBM_File,$WT::pass::passDB,O_RDWR,0) {
		&flocker::flockUN($WT::pass::passDB);
		&main::ErrServ("cannot access passDB");
		&main::LogWarn("could not access passDB: $!");
		&main::End;
	};
	$passDB{"pw","$uid\0"}=crypt($pw,&NewSalt);
	untie %passDB;
	&flocker::flockUN($WT::pass::passDB);
}

### public routines ###
# returns number of users
# since this is the first use of passDB during a WebThread run,
# initialize count if not already done
#######################
sub main::NumUsers {
	my(%passDB);
	&flocker::flockEX($WT::pass::passDB);
	unless (tie %passDB,SDBM_File,$WT::pass::passDB,O_RDWR|O_CREAT,$WT::pass::dbperms) {
		&flocker::flockUN($WT::pass::passDB);
		&main::ErrServ("cannot access passDB");
		&main::LogWarn("could not access passDB: $!");
		&main::End;
	};
	my($count)=($passDB{"count\0"} || ($passDB{"count\0"}=0));
	untie %passDB;
	&flocker::flockUN($WT::pass::passDB);
	$count;
}

### public routines ###
# returns uid for given uname
#######################
sub main::LookupUID {
	my(%passDB);
	&flocker::flockSH($WT::pass::passDB);
	unless (tie %passDB,SDBM_File,$WT::pass::passDB,O_RDONLY,0) {
		&flocker::flockUN($WT::pass::passDB);
		&main::ErrServ("cannot access passDB");
		&main::LogWarn("could not access passDB: $!");
		&main::End;
	};
	my($uid)=($passDB{"uid","$_[0]\0"} || 0);
	untie %passDB;
	&flocker::flockUN($WT::pass::passDB);
	$uid;
}

### public routines ###
# returns uname for given uid
#######################
sub main::LookupUname {
	my(%passDB);
	&flocker::flockSH($WT::pass::passDB);
	unless (tie %passDB,SDBM_File,$WT::pass::passDB,O_RDONLY,0) {
		&flocker::flockUN($WT::pass::passDB);
		&main::ErrServ("cannot access passDB");
		&main::LogWarn("could not access passDB: $!");
		&main::End;
	};
	my($uname)=$passDB{"uname","$_[0]\0"};
	untie %passDB;
	&flocker::flockUN($WT::pass::passDB);
	$uname;
}

### public routines ###
# user list iterator
#######################
sub main::EachUser {	# returns (uid,uname) for next user; uid=0 at end
	my($uname);
		# gotta use %WT::pass::passDB instead of my(%passDB)
		# since is must persist between calls to this routine
	unless ($WT::pass::iter) {	# starting new iteration?
		&flocker::flockSH($WT::pass::passDB);
		unless (tie %WT::pass::passDB,SDBM_File,$WT::pass::passDB,O_RDONLY,0) {
			&flocker::flockUN($WT::pass::passDB);
			&main::ErrServ("cannot access passDB");
			&main::LogWarn("could not access passDB: $!");
			&main::End;
		};
	};

	$WT::pass::iter++;		# prepare to get next user

	if ($WT::pass::iter>$WT::pass::passDB{"count\0"}) {	# no more users?
		untie %WT::pass::passDB;
		&flocker::flockUN($WT::pass::passDB);
		($WT::pass::iter,$uname)=(0,'');
	} else {
		$uname=$WT::pass::passDB{"uname","$WT::pass::iter\0"};
	};

	($WT::pass::iter,$uname);
}

### public routines ###
# adds a user to the passDB
#######################
sub main::AddUserPass {		# adds uname $_[0], pw $_[1], returns new uid
	my($uname,$pw)=@_;
	my(%passDB);

	&flocker::flockEX($WT::pass::passDB);
	unless (tie %passDB,SDBM_File,$WT::pass::passDB,O_CREAT|O_RDWR,$WT::pass::dbperms) {
		&flocker::flockUN($WT::pass::passDB);
		&main::ErrServ("cannot access passDB");
		&main::LogWarn("could not access passDB: $!");
		&main::End;
	};

	my($uid)=(++$passDB{"count\0"});

	$passDB{"uname","$uid\0"}=$uname;
	$passDB{"uid","$uname\0"}=$uid;
	$passDB{"pw","$uid\0"}=crypt($pw,&NewSalt);

	untie %passDB;
	&flocker::flockUN($WT::pass::passDB);

	$uid;
}

### private routines ###
# create a salt (taken from camel perl book, p188 "functions: srand"
########################
sub NewSalt {
	my @saltset = ('a'..'z','A'..'Z','0'..'9','.','/');
	srand( time | $$ );
	$saltset[rand(@saltset)].$saltset[rand(@saltset)];
}


1;
