package flocker;

# a way to flock() files by name
# $Id: flocker.pm,v 1.4 1996/08/14 14:11:28 dmacks Stab $

# hacked together thingy by Daniel Macks

# Copyright (c) 1996 by Daniel Macks. All rights reserved. 
# This library is free software; you may redistribute it 
# and/or modify it under the same terms as Perl itself.

use Carp;

### constructor ###
# package variables store info for all locks
###################
BEGIN {
	%flocker::lockfileFH=();
	$flocker::numLockfiles=0;
	$flocker::msg='';
	$flocker::no_flock=(-d ':');	# MacPerl5 doesn't have flock() yet
	$flocker::fake_it=0;		#if no_flock, then use semaphore file?
}

### private methods ###
# declare flock() constants
#######################
sub LOCK_SH { 1 }
sub LOCK_EX { 2 }
sub LOCK_NB { 4 }
sub LOCK_UN { 8 }

### private methods ###
# adds name of routine that called it to the passed error message
# saves error message in package variable and returns it
#######################
sub oy {
	$flocker::msg=(caller(1))[3];
	$flocker::msg.=": $_[0]" if $_[0];
	$flocker::msg;
}

### public methods ###
# add a new file to the list of lockfiles
######################
sub NewLockfile {
	return 1 if $flocker::no_flock;

	my $fn = shift or croak &flocker::oy("no filename given");
	my $lf = "$fn.lck";
	$flocker::lockfileFH{$lf}="flocker::LOCKFILE_".++$flocker::numLockfiles
		unless defined $flocker::lockfileFH{$lf};
	my $fh = $flocker::lockfileFH{$lf};

	unless (-e $lf) {		# make sure lockfile exists
		unless ( open($fh,">$lf") && close($fh) ) {
			&flocker::oy("could not create lockfile $lf: $!");
			return 0;
		};
	};

	unless (-r $lf && -w _) {	# make sure we can access lockfile
		&flocker::oy("bad permissions on lockfile $lf");
		return 0;
	};

	1;	# everything hunky-dory
}

### public methods ###
# blocking lock a file for shared (read-only) access
######################
sub flockSH {
	my $fn = shift or croak &flocker::oy("no filename given");
	my $lf = "${fn}.lck";
	my $fh = $flocker::lockfileFH{$lf}
		or croak &flocker::oy("unknown file $fn");

	return &flocker::fake_flock_lock($fn,$fh) if $flocker::no_flock;
	open($fh,"+>>$lf") or croak &flocker::oy("$lf: $!");
	flock($fh,LOCK_SH);
}

### public methods ###
# blocking lock a file for shared (read-only) access
######################
sub flockEX {
	my $fn = shift or croak &flocker::oy("no filename given");
	my $lf = "${fn}.lck";
	my $fh = $flocker::lockfileFH{$lf}
		or croak &flocker::oy("unknown file $fn");

	return &flocker::fake_flock_lock($fn,$fh) if $flocker::no_flock;
	open($fh,"+>>$lf") or croak &flocker::oy("$lf: $!");
	flock($fh,LOCK_EX);
}

### public methods ###
# unlock a file
######################
sub flockUN {
	my $fn = shift or croak &flocker::oy("no filename given");
	my $lf = "${fn}.lck";
	my $fh = $flocker::lockfileFH{$lf}
		or croak &flocker::oy("unknown file $fn");

	return &flocker::fake_flock_unlock($fn,$fh) if $flocker::no_flock;
	flock($fh,LOCK_UN);
	close($fh);
}

### private methods ###
# fake a flock by using the presence/absence of a file as a semaphore
# these locks are always exclusive, and they don't die when the process does
#######################
sub fake_flock_lock {
	return 1 unless $flocker::fake_it;

	my $fn = shift;
	my $sem = "${fn}.sem";

	while (-e $sem) {			# check for lock
		select(undef,undef,undef,0.1);	# wait 1/10 second
	};					# try again forever

	open(FAKE_FLOCK,">$sem") || croak &flocker::oy("failed open $sem: $!");
	close(FAKE_FLOCK) || croak &flocker::oy("failed close $sem: $!");
	1;
}

sub fake_flock_unlock {
	return 1 unless $flocker::fake_it;
	my $fn = shift;
	my $sem = "${fn}.sem";

	unlink $sem || croak &flocker::oy("failed unlink $sem: $!");
	1;
}

1;

